#include "OLIMEX_I2C.h"
#include "Arduino.h"

void C_OLIMEX_I2C::StartI2C(void)
{
	pinMode(A4, OUTPUT);
	delay(2*PER);
	pinMode(A5, OUTPUT);
	delay(PER);
}
void C_OLIMEX_I2C::StopI2C(void)
{

	delay(2*PER);
	pinMode(A4, OUTPUT);
	delay(PER);
	pinMode(A5,  INPUT);
	delay(2*PER);
	pinMode(A4, INPUT);
	delay(10*PER);
}
char C_OLIMEX_I2C::WriteByteI2C(char data)
{
	int i, ack, ch;
	for(i = 0; i < 8; i++)
	{
		delay(PER);
		if(data & 0x80)
			pinMode(A4, INPUT);
		else
			pinMode(A4, OUTPUT);
		data <<= 1;
		delay(PER);
		pinMode(A5,  INPUT);
		delay(2*PER);
		pinMode(A5, OUTPUT);
	}

//-------READ ACK---------
	delay(PER);
	pinMode(A4, INPUT);
	delay(PER);
	pinMode(A5,  INPUT);
	delay(PER);
	ack = digitalRead(A4);
	delay(PER);
	pinMode(A5, OUTPUT);
	delay(PER);

	//pinMode(A4, OUTPUT);	//GSP
	
	return ack;

}
char C_OLIMEX_I2C::ReadByteI2C(void)
{
	char data = 0;
	char i;
	pinMode(A4, INPUT);
	for(i = 0; i < 8; i++)
	{
		delay(PER);
		pinMode(A5,  INPUT);
		while(digitalRead(A5) == 0);	//A5 STRETCHING
		delay(PER);
		data |= digitalRead(A4);
		if(i != 7) data <<= 1;
		delay(PER);
		pinMode(A5, OUTPUT);
		delay(PER);
	}
	return data;
}
short C_OLIMEX_I2C::ReadWordI2C(void)
{
	int data = 0;
	int i;
	pinMode(A4, INPUT);
	for(i = 0; i < 16; i++)
	{
		delay(PER);
		pinMode(A5, INPUT);
		while(digitalRead(A5) == 0);	//CLOCK STRETCHING
		delay(PER);
		data |= digitalRead(A4);
		if(i != 15) data <<= 1;
		delay(PER);
		pinMode(A5, OUTPUT);
		delay(PER);


	}
	return data;
}
long C_OLIMEX_I2C::ReadLongI2C(void)
{
	
	long int data = 0;
	int i;
	pinMode(A4, INPUT);
	for(i = 0; i < 32; i++)
	{
		delay(PER);
		pinMode(A5, INPUT);
		while(digitalRead(A5) == 0);	//CLOCK STRETCHING
		delay(PER);
		data |= digitalRead(A4);
		if(i != 31) data <<= 1;
		delay(PER);
		pinMode(A5, OUTPUT);
		delay(PER);


	}
	return data;
}
char C_OLIMEX_I2C::ScanAddress(char address)
{
	char ack;
	StartI2C();
	WriteByteI2C(0x00);
	StopI2C();
	StartI2C();
	WriteByteI2C(company);
	WriteByteI2C(device);
	ack = WriteByteI2C(address);
	WriteByteI2C(0xFF);
	StopI2C();


	return ack;


}
char C_OLIMEX_I2C::SetAddress()
{
	char ack, buffer[10];
	unsigned char addr;

	Serial.println("");
	Serial.print("SET NEW ADDRESS > ");

	ReadCommand(buffer);

	if(buffer[0] == '\0')
	{
		for (int i = 0; i < 0x10; i++)
		{
			addr = 0xA0 + i;
			if(ScanAddress(addr))
				break;
		}
		Serial.write("\t\t\t");
		Serial.print ("0x");
		Serial.println(addr, HEX);

		StartI2C();
		WriteByteI2C(company);
		WriteByteI2C(device);
		WriteByteI2C(ADR_DFL);
		WriteByteI2C(0xB0);
		ack = WriteByteI2C(addr);
		StopI2C();


		Serial.println("");
		if(ack)
			Serial.println("ERROR!!! There are no devices to program.");
		else
			Serial.println("DONE.");
	}

	else
	{
		addr = StringToInt(buffer);

		StartI2C();
		WriteByteI2C(company);
		WriteByteI2C(device);
		WriteByteI2C(ADR_DFL);
		WriteByteI2C(0xB0);
		ack = WriteByteI2C(addr);
		StopI2C();
		Serial.println("");
		if(ack)
			Serial.println("ERROR!!! There are no devices to program.");
		else
			Serial.println("DONE.");

	}
	return ack;
}
unsigned char C_OLIMEX_I2C::StringToInt(char Str[])
{
	long int Temp=0;
	int i, First=0, System=10, Digit;
  
  
	// check for binary or hexadecimal format
	if (Str[0] == '0' && (Str[1] == 'b' || Str[1] == 'B' || Str[1] == 'x' || Str[1] == 'X'))
		{
		if (Str[1] == 'b' || Str[1] == 'B')
			System = 2;
		else
			System = 16;
		First = 2;
		}
  
	// convert string to binary/decimal/hexadecimal
  switch (System)
  {
  case 2:
    for (i=First; Str[i]; i++)
    {
      Digit = Str[i];
      if (Digit >= '0' && Digit<= '1')
        Temp = Temp*2 + (Digit & 0x0F);
    }
    break;
  case 10:
    for (i=First; Str[i]; i++)
    {
      Digit = Str[i];
      if (Digit >= '0' && Digit<= '9')
        Temp = Temp*10 + (Digit & 0x0F);
    }
    break;
  case 16:
    for (i=First; Str[i]; i++)
    {
      Digit = Str[i];
      if (Digit >= '0' && Digit <= '9')
        Digit = Digit - '0';
      if (Digit >= 'a' && Digit <= 'f')
        Digit = Digit - 'a' + 10;
      if (Digit >= 'A' && Digit <= 'F')
        Digit = Digit - 'A' + 10;
      Temp = Temp * 16 + Digit;
    }
    break;
  }
  
  return Temp;
}
void C_OLIMEX_I2C::ReadCommand(char *buf)
{
	char ch, i = 0, buffer[30];
	while(1)
		{
			if(Serial.available() > 0 )
			{
				ch = Serial.read();
				if(ch == 0x7F)
				{
					if(i > 0) i--;
					continue;
				}
				if(ch == '\n' || ch == '\r')
				{
					buffer[i] = '\0';
					break;
				}
				else
				{
					buffer[i] =  ch;
				}
				i++;
			}
		}
	strcpy(buf, buffer);

}